@@ -1,5 +1,5 @@
 /*
-    $Id: sector.h,v 1.13 2004/06/02 00:43:53 rocky Exp $
+    $Id: sector.h,v 1.28 2004/08/30 01:01:14 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -67,6 +67,59 @@
 
 #include <cdio/types.h>
 
+/*! track modes (Table 350) 
+   reference: MMC-3 draft revsion - 10g
+*/
+typedef enum {
+	AUDIO,				/**< 2352 byte block length */
+	MODE1,				/**< 2048 byte block length */
+	MODE1_RAW,			/**< 2352 byte block length */
+	MODE2,				/**< 2336 byte block length */
+	MODE2_FORM1,			/**< 2048 byte block length */
+	MODE2_FORM2,			/**< 2324 byte block length */
+	MODE2_FORM_MIX,			/**< 2336 byte block length */
+	MODE2_RAW			/**< 2352 byte block length */
+} trackmode_t;
+
+/*! disc modes. The first combined from MMC-3 5.29.2.8 (Send CUESHEET)
+  and GNU/Linux /usr/include/linux/cdrom.h and we've added DVD.
+ */
+typedef enum {
+	CDIO_DISC_MODE_CD_DA,		/**< CD-DA */
+	CDIO_DISC_MODE_CD_DATA,	        /**< CD-ROM form 1 */
+	CDIO_DISC_MODE_CD_XA,	        /**< CD-ROM XA form2 */
+	CDIO_DISC_MODE_CD_MIXED,	/**< Some combo of above. */
+        CDIO_DISC_MODE_DVD_ROM,         /**< DVD ROM (e.g. movies) */
+        CDIO_DISC_MODE_DVD_RAM,         /**< DVD-RAM */
+        CDIO_DISC_MODE_DVD_R,           /**< DVD-R */
+        CDIO_DISC_MODE_DVD_RW,          /**< DVD-RW */
+        CDIO_DISC_MODE_DVD_PR,          /**< DVD+R */
+        CDIO_DISC_MODE_DVD_PRW,         /**< DVD+RW */
+        CDIO_DISC_MODE_DVD_OTHER,       /**< Unknown/unclassified DVD type */
+        CDIO_DISC_MODE_NO_INFO,
+        CDIO_DISC_MODE_ERROR
+} discmode_t;
+
+/*! Information that can be obtained through a Read Subchannel
+    command.
+ */
+#define CDIO_SUBCHANNEL_SUBQ_DATA		0
+#define CDIO_SUBCHANNEL_CURRENT_POSITION	1
+#define CDIO_SUBCHANNEL_MEDIA_CATALOG	        2
+#define CDIO_SUBCHANNEL_TRACK_ISRC		3
+
+/*! track flags
+ * Q Sub-channel Control Field (4.2.3.3)
+ */
+typedef enum {
+	NONE = 			0x00,	/* no flags set */
+	PRE_EMPHASIS =		0x01,	/* audio track recorded with pre-emphasis */
+	COPY_PERMITTED =	0x02,	/* digital copy permitted */
+	DATA =			0x04,	/* data track */
+	FOUR_CHANNEL_AUDIO =	0x08,	/* 4 audio channels */
+	SCMS =			0x10	/* SCMS (5.29.2.7) */
+} flag_t;
+
 #define CDIO_PREGAP_SECTORS  150
 #define CDIO_POSTGAP_SECTORS 150
 
@@ -119,16 +172,31 @@
 #define	CDIO_CDROM_CDI_TRACK	0x10
 #define	CDIO_CDROM_XA_TRACK	0x20
 
-/*! The leadout track is always 0xAA, regardless of # of tracks on disc */
+/*! The leadout track is always 0xAA, regardless of # of tracks on
+    disc, or what value may be used internally. For example although
+    OS X uses a different value for the lead-out track internally than
+    given below, programmers should use CDIO_CDROM_LEADOUT_TRACK and
+    not worry about this.
+ */
 #define	CDIO_CDROM_LEADOUT_TRACK 0xAA
 
 #define M2F2_SECTOR_SIZE    2324
 #define M2SUB_SECTOR_SIZE   2332
 #define M2RAW_SECTOR_SIZE   2336
 
+/*! Largest CD track number */
 #define CDIO_CD_MAX_TRACKS    99 
+/*! Smallest CD track number */
 #define CDIO_CD_MIN_TRACK_NO   1
+
+/*! Largest CD session number */
+#define CDIO_CD_MAX_SESSIONS    99 
+/*! Smallest CD session number */
+#define CDIO_CD_MIN_SESSION_NO   1
+
+/*! Largest LSN in a CD */
 #define CDIO_CD_MAX_LSN   450150
+/*! Smallest LSN in a CD */
 #define CDIO_CD_MIN_LSN  -450150
 
 
@@ -166,6 +234,7 @@ void  cdio_lba_to_msf(lba_t lba, msf_t *msf);
 
 /*! 
   Convert an LSN into the corresponding LBA.
+  CDIO_INVALID_LBA is returned if there is an error.
 */
 lba_t cdio_lsn_to_lba (lsn_t lsn);
 
@@ -176,20 +245,75 @@ void  cdio_lsn_to_msf (lsn_t lsn, msf_t *msf);
 
 /*! 
   Convert a MSF into the corresponding LBA.
+  CDIO_INVALID_LBA is returned if there is an error.
 */
-lba_t
-cdio_msf_to_lba (const msf_t *msf);
+lba_t cdio_msf_to_lba (const msf_t *msf);
 
 /*! 
   Convert a MSF into the corresponding LSN.
+  CDIO_INVALID_LSN is returned if there is an error.
 */
-lsn_t
-cdio_msf_to_lsn (const msf_t *msf);
+lsn_t cdio_msf_to_lsn (const msf_t *msf);
+
+/*!  
+  Convert a MSF - broken out as 3 integer components into the
+  corresponding LBA.  
+  CDIO_INVALID_LBA is returned if there is an error.
+*/
+lba_t cdio_msf3_to_lba (unsigned int minutes, unsigned int seconds, 
+                        unsigned int frames);
+      
+/*! 
+  Convert a string of the form MM:SS:FF into the corresponding LBA.
+  CDIO_INVALID_LBA is returned if there is an error.
+*/
+lba_t cdio_mmssff_to_lba (const char *psz_mmssff);
+      
+/*! 
+  Return true if discmode is some sort of CD.
+*/
+bool cdio_is_discmode_cdrom (discmode_t discmode);
+      
+/*! 
+  Return true if discmode is some sort of DVD.
+*/
+bool cdio_is_discmode_dvd (discmode_t discmode);
+      
 
 #ifdef __cplusplus
     }
 #endif
 
+static inline bool discmode_is_cd(discmode_t discmode) 
+{
+  switch (discmode) {
+  case CDIO_DISC_MODE_CD_DA:
+  case CDIO_DISC_MODE_CD_DATA:
+  case CDIO_DISC_MODE_CD_XA:
+  case CDIO_DISC_MODE_CD_MIXED:
+    return true;
+  default: 
+    return false;
+  }
+}
+
+static inline bool discmode_is_dvd(discmode_t discmode) 
+{
+  switch (discmode) {
+  case CDIO_DISC_MODE_DVD_ROM:
+  case CDIO_DISC_MODE_DVD_RAM:
+  case CDIO_DISC_MODE_DVD_R:
+  case CDIO_DISC_MODE_DVD_RW:
+  case CDIO_DISC_MODE_DVD_PR:
+  case CDIO_DISC_MODE_DVD_PRW:
+  case CDIO_DISC_MODE_DVD_OTHER:
+    return true;
+  default: 
+    return false;
+  }
+}
+
+
 #endif /* _CDIO_SECTOR_H_ */
 
 
