@@ -1,5 +1,5 @@
 /*
-    $Id: win32.c,v 1.45 2004/08/27 04:12:29 rocky Exp $
+    $Id: win32.c,v 1.50 2004/10/31 17:43:30 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
 
@@ -26,7 +26,7 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: win32.c,v 1.45 2004/08/27 04:12:29 rocky Exp $";
+static const char _rcsid[] = "$Id: win32.c,v 1.50 2004/10/31 17:43:30 rocky Exp $";
 
 #include <cdio/cdio.h>
 #include <cdio/sector.h>
@@ -41,25 +41,56 @@ static const char _rcsid[] = "$Id: win32.c,v 1.45 2004/08/27 04:12:29 rocky Exp
 
 #include <ctype.h>
 #include <stdio.h>
+
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
 
 #include <windows.h>
 #include <winioctl.h>
 #include "win32.h"
 
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
-#include <sys/types.h>
-#include "aspi32.h"
+#endif
 
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#if defined (MSVC) || defined (_XBOX)
+#undef IN
+#else
+#include "aspi32.h"
+#endif
+
+#ifdef _XBOX
+#include "stdint.h"
+#include <xtl.h>
+#define WIN_NT 1
+#else
 #define WIN_NT               ( GetVersion() < 0x80000000 )
+#endif
 
 /* General ioctl() CD-ROM command function */
 static bool 
 _cdio_mciSendCommand(int id, UINT msg, DWORD flags, void *arg)
 {
+#ifdef _XBOX
+  return false;
+#else
   MCIERROR mci_error;
   
   mci_error = mciSendCommand(id, msg, flags, (DWORD)arg);
@@ -70,6 +101,7 @@ _cdio_mciSendCommand(int id, UINT msg, DWORD flags, void *arg)
     cdio_warn("mciSendCommand() error: %s", error);
   }
   return(mci_error == 0);
+#endif
 }
 
 static access_mode_t 
@@ -82,10 +114,16 @@ str_to_access_mode_win32(const char *psz_access_mode)
   
   if (!strcmp(psz_access_mode, "ioctl"))
     return _AM_IOCTL;
-  else if (!strcmp(psz_access_mode, "ASPI"))
+  else if (!strcmp(psz_access_mode, "ASPI")) {
+#ifdef _XBOX
     return _AM_ASPI;
-  else {
-    cdio_warn ("unknown access type: %s. Default used.", 
+#else 
+    cdio_warn ("XBOX doesn't support access type: %s. Default used instead.", 
+	       psz_access_mode);
+    return default_access_mode;
+#endif    
+  } else {
+    cdio_warn ("unknown access type: %s. Default used instead.", 
 	       psz_access_mode);
     return default_access_mode;
   }
@@ -402,7 +440,9 @@ read_toc_win32 (void *p_user_data)
  */
 static int 
 _cdio_eject_media (void *user_data) {
-
+#ifdef _XBOX
+  return -1;
+#else
   _img_private_t *env = user_data;
 
 
@@ -432,6 +472,7 @@ _cdio_eject_media (void *user_data) {
     ret = 0;
   
   return ret;
+#endif
 }
 
 /*!
@@ -475,13 +516,16 @@ _cdio_get_mcn (const void *p_user_data) {
   Get format of track. 
 */
 static track_format_t
-_cdio_get_track_format(void *obj, track_t i_track) 
+_cdio_get_track_format(void *p_obj, track_t i_track) 
 {
-  _img_private_t *p_env = obj;
+  _img_private_t *p_env = p_obj;
   
-  if ( NULL == p_env ||
-       ( i_track < p_env->gen.i_first_track
-	 || i_track >= p_env->gen.i_tracks + p_env->gen.i_first_track ) )
+  if ( !p_env ) return TRACK_FORMAT_ERROR;
+  
+  if (!p_env->gen.toc_init) read_toc_win32 (p_env) ;
+
+  if ( i_track < p_env->gen.i_first_track
+       || i_track >= p_env->gen.i_tracks + p_env->gen.i_first_track )
     return TRACK_FORMAT_ERROR;
 
   if( p_env->hASPI ) {
@@ -680,34 +724,37 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
   _img_private_t *_data;
   char *psz_source;
 
-  cdio_funcs _funcs = {
-    .eject_media        = _cdio_eject_media,
-    .free               = _free_win32,
-    .get_arg            = _get_arg_win32,
-    .get_cdtext         = get_cdtext_generic,
-    .get_default_device = cdio_get_default_device_win32,
-    .get_devices        = cdio_get_devices_win32,
-    .get_discmode       = get_discmode_win32,
-    .get_drive_cap      = scsi_mmc_get_drive_cap_generic,
-    .get_first_track_num= get_first_track_num_generic,
-    .get_mcn            = _cdio_get_mcn, 
-    .get_num_tracks     = get_num_tracks_generic,
-    .get_track_format   = _cdio_get_track_format,
-    .get_track_green    = _cdio_get_track_green,
-    .get_track_lba      = NULL, /* This could be implemented if need be. */
-    .get_track_msf      = _cdio_get_track_msf,
-    .lseek              = NULL,
-    .read               = NULL,
-    .read_audio_sectors = _cdio_read_audio_sectors,
-    .read_mode1_sector  = _cdio_read_mode1_sector,
-    .read_mode1_sectors = _cdio_read_mode1_sectors,
-    .read_mode2_sector  = _cdio_read_mode2_sector,
-    .read_mode2_sectors = _cdio_read_mode2_sectors,
-    .read_toc           = &read_toc_win32,
-    .run_scsi_mmc_cmd   = &run_scsi_cmd_win32,
-    .set_arg            = set_arg_win32,
-    .stat_size          = stat_size_win32
-  };
+  cdio_funcs _funcs;
+
+  memset( &_funcs, 0, sizeof(_funcs) );
+
+  _funcs.eject_media        = _cdio_eject_media;
+  _funcs.free               = _free_win32;
+  _funcs.get_arg            = _get_arg_win32;
+  _funcs.get_cdtext         = get_cdtext_generic;
+  _funcs.get_default_device = cdio_get_default_device_win32;
+  _funcs.get_devices        = cdio_get_devices_win32;
+  _funcs.get_discmode       = get_discmode_win32;
+  _funcs.get_drive_cap      = scsi_mmc_get_drive_cap_generic;
+  _funcs.get_first_track_num= get_first_track_num_generic;
+  _funcs.get_hwinfo         = NULL;
+  _funcs.get_mcn            = _cdio_get_mcn;
+  _funcs.get_num_tracks     = get_num_tracks_generic;
+  _funcs.get_track_format   = _cdio_get_track_format;
+  _funcs.get_track_green    = _cdio_get_track_green;
+  _funcs.get_track_lba      = NULL; /* This could be implemented if need be. */
+  _funcs.get_track_msf      = _cdio_get_track_msf;
+  _funcs.lseek              = NULL;
+  _funcs.read               = NULL;
+  _funcs.read_audio_sectors = _cdio_read_audio_sectors;
+  _funcs.read_mode1_sector  = _cdio_read_mode1_sector;
+  _funcs.read_mode1_sectors = _cdio_read_mode1_sectors;
+  _funcs.read_mode2_sector  = _cdio_read_mode2_sector;
+  _funcs.read_mode2_sectors = _cdio_read_mode2_sectors;
+  _funcs.read_toc           = &read_toc_win32;
+  _funcs.run_scsi_mmc_cmd   = &run_scsi_cmd_win32;
+  _funcs.set_arg            = set_arg_win32;
+  _funcs.stat_size          = stat_size_win32;
 
   _data                 = _cdio_malloc (sizeof (_img_private_t));
   _data->access_mode    = str_to_access_mode_win32(psz_access_mode);
