@@ -1,5 +1,5 @@
 /*
-    $Id: win32.h,v 1.7 2004/06/25 01:47:06 rocky Exp $
+    $Id: win32.h,v 1.21 2004/08/10 03:44:56 rocky Exp $
 
     Copyright (C) 2004 Rocky Bernstein <rocky@panix.com>
 
@@ -26,6 +26,7 @@ typedef struct {
   lsn_t          start_lsn;
   UCHAR          Control : 4;
   UCHAR          Format;
+  cdtext_t       cdtext;	         /* CD-TEXT */
 } track_info_t;
 
 typedef enum {
@@ -41,8 +42,9 @@ typedef struct {
 
   access_mode_t access_mode;
 
-  bool b_ioctl_init;
-  bool b_aspi_init;
+  /* Some of the more OS specific things. */
+    /* Entry info for each track, add 1 for leadout. */
+  track_info_t  tocent[CDIO_CD_MAX_TRACKS+1];
 
   HANDLE h_device_handle; /* device descriptor */
   long  hASPI;
@@ -50,14 +52,16 @@ typedef struct {
   short i_lun;
   long  (*lpSendCommand)( void* );
 
-  /* Track information */
-  bool toc_init;                 /* if true, info below is valid. */
-  track_info_t  tocent[100];     /* entry info for each track */
-  track_t       total_tracks;    /* number of tracks in image */
-  track_t       i_first_track;   /* track number of first track */
+  bool b_ioctl_init;
+  bool b_aspi_init;
 
 } _img_private_t;
 
+/*! 
+  Get disc type associated with cd object.
+*/
+discmode_t get_discmode_win32ioctl (_img_private_t *p_env);
+
 /*!
    Reads an audio device using the DeviceIoControl method into data
    starting from lsn.  Returns 0 if no error.
@@ -80,6 +84,28 @@ int read_mode1_sector_win32ioctl (const _img_private_t *env, void *data,
 
 const char *is_cdrom_win32ioctl (const char drive_letter);
 
+/*!
+  Run a SCSI MMC command. 
+ 
+  env	        private CD structure 
+  i_timeout_ms  time in milliseconds we will wait for the command
+                to complete. If this value is -1, use the default 
+		time-out value.
+  p_buf	        Buffer for data, both sending and receiving
+  i_buf	        Size of buffer
+  e_direction	direction the transfer is to go.
+  cdb	        CDB bytes. All values that are needed should be set on 
+                input. We'll figure out what the right CDB length should be.
+
+  Return 0 if command completed successfully.
+ */
+int run_scsi_cmd_win32ioctl( const void *p_user_data, 
+			     unsigned int i_timeout,
+			     unsigned int i_cdb, 
+			     const scsi_mmc_cdb_t * p_cdb,
+			     scsi_mmc_direction_t e_direction, 
+			     unsigned int i_buf, /*in/out*/ void *p_buf );
+
 /*!
   Initialize internal structures for CD device.
  */
@@ -91,16 +117,22 @@ bool init_win32ioctl (_img_private_t *env);
 */
 bool read_toc_win32ioctl (_img_private_t *env);
 
+/*!
+  Return the media catalog number MCN.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
 char *get_mcn_win32ioctl (const _img_private_t *env);
 
-/*!
-  Return the the kind of drive capabilities of device.
-
-  Note: string is malloc'd so caller should free() then returned
-  string when done with it.
-
- */
-cdio_drive_cap_t get_drive_cap_aspi (const _img_private_t *env);
+/*
+  Read cdtext information for a CdIo object .
+  
+  return true on success, false on error or CD-TEXT information does
+  not exist.
+*/
+bool init_cdtext_win32ioctl (_img_private_t *env);
 
 /*!
   Return the the kind of drive capabilities of device.
@@ -109,10 +141,30 @@ cdio_drive_cap_t get_drive_cap_aspi (const _img_private_t *env);
   string when done with it.
 
  */
-cdio_drive_cap_t get_drive_cap_win32ioctl (const _img_private_t *env);
+void get_drive_cap_aspi (const _img_private_t *env,
+			 cdio_drive_read_cap_t  *p_read_cap,
+			 cdio_drive_write_cap_t *p_write_cap,
+			 cdio_drive_misc_cap_t  *p_misc_cap);
+
+/*!
+  Return the the kind of drive capabilities of device.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+void get_drive_cap_win32ioctl (const _img_private_t *env,
+			       cdio_drive_read_cap_t  *p_read_cap,
+			       cdio_drive_write_cap_t *p_write_cap,
+			       cdio_drive_misc_cap_t  *p_misc_cap);
 
 /*!  
   Get the format (XA, DATA, AUDIO) of a track. 
 */
 track_format_t get_track_format_win32ioctl(const _img_private_t *env, 
 					   track_t i_track); 
+
+void set_cdtext_field_win32(void *user_data, track_t i_track, 
+			    track_t i_first_track,
+			    cdtext_field_t e_field, const char *psz_value);
+
