@@ -1,5 +1,5 @@
 /*
-    $Id: _cdio_bsdi.c,v 1.41 2004/08/16 01:47:49 rocky Exp $
+    $Id: _cdio_bsdi.c,v 1.44 2004/11/20 12:50:26 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -27,8 +27,9 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: _cdio_bsdi.c,v 1.41 2004/08/16 01:47:49 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_bsdi.c,v 1.44 2004/11/20 12:50:26 rocky Exp $";
 
+#include <cdio/logging.h>
 #include <cdio/sector.h>
 #include <cdio/util.h>
 #include "cdio_assert.h"
@@ -230,9 +231,9 @@ _read_audio_sectors_bsdi (void *user_data, void *data, lsn_t lsn,
   _img_private_t *p_env = user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
-  msf->cdmsf_min0 = from_bcd8(_msf.m);
-  msf->cdmsf_sec0 = from_bcd8(_msf.s);
-  msf->cdmsf_frame0 = from_bcd8(_msf.f);
+  msf->cdmsf_min0 = cdio_from_bcd8(_msf.m);
+  msf->cdmsf_sec0 = cdio_from_bcd8(_msf.s);
+  msf->cdmsf_frame0 = cdio_from_bcd8(_msf.f);
 
   if (p_env->gen.ioctls_debugged == 75)
     cdio_debug ("only displaying every 75th ioctl from now on");
@@ -329,9 +330,9 @@ _read_mode2_sector_bsdi (void *user_data, void *data, lsn_t lsn,
   _img_private_t *p_env = user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
-  msf->cdmsf_min0 = from_bcd8(_msf.m);
-  msf->cdmsf_sec0 = from_bcd8(_msf.s);
-  msf->cdmsf_frame0 = from_bcd8(_msf.f);
+  msf->cdmsf_min0 = cdio_from_bcd8(_msf.m);
+  msf->cdmsf_sec0 = cdio_from_bcd8(_msf.s);
+  msf->cdmsf_frame0 = cdio_from_bcd8(_msf.f);
 
   if (p_env->gen.ioctls_debugged == 75)
     cdio_debug ("only displaying every 75th ioctl from now on");
@@ -669,9 +670,9 @@ _get_track_msf_bsdi(void *user_data, track_t i_track, msf_t *msf)
 
   {
     struct cdrom_msf0  *msf0= &p_env->tocent[i_track].cdte_addr.msf;
-    msf->m = to_bcd8(msf0->minute);
-    msf->s = to_bcd8(msf0->second);
-    msf->f = to_bcd8(msf0->frame);
+    msf->m = cdio_to_bcd8(msf0->minute);
+    msf->s = cdio_to_bcd8(msf0->second);
+    msf->f = cdio_to_bcd8(msf0->frame);
     return true;
   }
 }
@@ -773,6 +774,7 @@ cdio_open_bsdi (const char *psz_orig_source)
     .get_drive_cap      = scsi_mmc_get_drive_cap_generic,
     .get_discmode       = get_discmode_generic,
     .get_first_track_num= get_first_track_num_generic,
+    .get_hwinfo         = NULL,
     .get_mcn            = _get_mcn_bsdi, 
     .get_num_tracks     = get_num_tracks_generic,
     .get_track_format   = get_track_format_bsdi,
@@ -811,7 +813,7 @@ cdio_open_bsdi (const char *psz_orig_source)
     else {
       /* The below would be okay if all device drivers worked this way. */
 #if 0
-      cdio_info ("source %s is a not a device", psz_orig_source);
+      cdio_info ("source %s is not a device", psz_orig_source);
 #endif
       return NULL;
     }
