@@ -1,8 +1,7 @@
 /*
-    $Id: _cdio_generic.c,v 1.25 2004/08/28 09:15:41 rocky Exp $
+    $Id: _cdio_generic.c,v 1.29 2004/11/01 09:14:21 rocky Exp $
 
-    Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2004 Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,15 +18,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* This file contains Linux-specific code and implements low-level 
-   control of the CD drive.
+/* This file contains generic implementations of device-dirver routines.
 */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: _cdio_generic.c,v 1.25 2004/08/28 09:15:41 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_generic.c,v 1.29 2004/11/01 09:14:21 rocky Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -49,6 +47,7 @@ static const char _rcsid[] = "$Id: _cdio_generic.c,v 1.25 2004/08/28 09:15:41 ro
 #include "cdio_assert.h"
 #include "cdio_private.h"
 #include "_cdio_stdio.h"
+#include "portable.h"
 
 /*!
   Eject media -- there's nothing to do here. We always return 2.
@@ -107,6 +106,7 @@ cdio_generic_init (void *user_data)
   p_env->toc_init = false;
   p_env->b_cdtext_init  = false;
   p_env->b_cdtext_error = false;
+  p_env->i_joliet_level = 0;  /* Assume no Joliet extensions initally */
   return true;
 }
 
@@ -198,7 +198,8 @@ cdio_is_device_quiet_generic(const char *source_name)
   Use cdio_free_device_list() to free this device_list.
 */
 void 
-cdio_add_device_list(char **device_list[], const char *drive, int *num_drives)
+cdio_add_device_list(char **device_list[], const char *drive, 
+		     unsigned int *num_drives)
 {
   if (NULL != drive) {
     unsigned int j;
