@@ -1,5 +1,5 @@
 /*
-    $Id: _cdio_sunos.c,v 1.74 2004/08/16 01:47:49 rocky Exp $
+    $Id: _cdio_sunos.c,v 1.79 2004/11/18 01:56:09 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -38,7 +38,7 @@
 
 #ifdef HAVE_SOLARIS_CDROM
 
-static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.74 2004/08/16 01:47:49 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.79 2004/11/18 01:56:09 rocky Exp $";
 
 #ifdef HAVE_GLOB_H
 #include <glob.h>
@@ -122,12 +122,12 @@ str_to_access_mode_sunos(const char *psz_access_mode)
   Initialize CD device.
  */
 static bool
-init_solaris (_img_private_t *env)
+init_solaris (_img_private_t *p_env)
 {
 
-  if (!cdio_generic_init(env)) return false;
+  if (!cdio_generic_init(p_env)) return false;
   
-  env->access_mode = _AM_SUN_CTRL_SCSI;    
+  p_env->access_mode = _AM_SUN_CTRL_SCSI;    
 
   return true;
 }
@@ -181,41 +181,41 @@ static int
 _read_audio_sectors_solaris (void *p_user_data, void *data, lsn_t lsn, 
 			  unsigned int nblocks)
 {
-  char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
-  struct cdrom_msf *msf = (struct cdrom_msf *) &buf;
+  struct cdrom_msf solaris_msf;
   msf_t _msf;
   struct cdrom_cdda cdda;
 
-  _img_private_t *env = p_user_data;
+  _img_private_t *p_env = p_user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
-  msf->cdmsf_min0   = from_bcd8(_msf.m);
-  msf->cdmsf_sec0   = from_bcd8(_msf.s);
-  msf->cdmsf_frame0 = from_bcd8(_msf.f);
+  solaris_msf.cdmsf_min0   = cdio_from_bcd8(_msf.m);
+  solaris_msf.cdmsf_sec0   = cdio_from_bcd8(_msf.s);
+  solaris_msf.cdmsf_frame0 = cdio_from_bcd8(_msf.f);
   
-  if (env->gen.ioctls_debugged == 75)
+  if (p_env->gen.ioctls_debugged == 75)
     cdio_debug ("only displaying every 75th ioctl from now on");
   
-  if (env->gen.ioctls_debugged == 30 * 75)
+  if (p_env->gen.ioctls_debugged == 30 * 75)
     cdio_debug ("only displaying every 30*75th ioctl from now on");
   
-  if (env->gen.ioctls_debugged < 75 
-      || (env->gen.ioctls_debugged < (30 * 75)  
-	  && env->gen.ioctls_debugged % 75 == 0)
-      || env->gen.ioctls_debugged % (30 * 75) == 0)
+  if (p_env->gen.ioctls_debugged < 75 
+      || (p_env->gen.ioctls_debugged < (30 * 75)  
+	  && p_env->gen.ioctls_debugged % 75 == 0)
+      || p_env->gen.ioctls_debugged % (30 * 75) == 0)
     cdio_debug ("reading %d", lsn);
   
-  env->gen.ioctls_debugged++;
+  p_env->gen.ioctls_debugged++;
   
-  cdda.cdda_addr   = lsn;
-  cdda.cdda_length = nblocks;
-  cdda.cdda_data   = (caddr_t) data;
-  if (ioctl (env->gen.fd, CDROMCDDA, &cdda) == -1) {
+  cdda.cdda_addr    = lsn;
+  cdda.cdda_length  = nblocks;
+  cdda.cdda_data    = (caddr_t) data;
+  cdda.cdda_subcode = CDROM_DA_NO_SUBCODE;
+  
+  if (ioctl (p_env->gen.fd, CDROMCDDA, &cdda) == -1) {
     perror ("ioctl(..,CDROMCDDA,..)");
 	return 1;
 	/* exit (EXIT_FAILURE); */
   }
-  memcpy (data, buf, CDIO_CD_FRAMESIZE_RAW);
   
   return 0;
 }
@@ -268,7 +268,7 @@ _read_mode2_sector_solaris (void *p_user_data, void *p_data, lsn_t lsn,
 			    bool b_form2)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
-  struct cdrom_msf *msf = (struct cdrom_msf *) &buf;
+  struct cdrom_msf solaris_msf;
   msf_t _msf;
   int offset = 0;
   struct cdrom_cdxa cd_read;
@@ -276,9 +276,9 @@ _read_mode2_sector_solaris (void *p_user_data, void *p_data, lsn_t lsn,
   _img_private_t *p_env = p_user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
-  msf->cdmsf_min0 = from_bcd8(_msf.m);
-  msf->cdmsf_sec0 = from_bcd8(_msf.s);
-  msf->cdmsf_frame0 = from_bcd8(_msf.f);
+  solaris_msf.cdmsf_min0   = cdio_from_bcd8(_msf.m);
+  solaris_msf.cdmsf_sec0   = cdio_from_bcd8(_msf.s);
+  solaris_msf.cdmsf_frame0 = cdio_from_bcd8(_msf.f);
   
   if (p_env->gen.ioctls_debugged == 75)
     cdio_debug ("only displaying every 75th ioctl from now on");
@@ -291,7 +291,8 @@ _read_mode2_sector_solaris (void *p_user_data, void *p_data, lsn_t lsn,
 	  && p_env->gen.ioctls_debugged % 75 == 0)
       || p_env->gen.ioctls_debugged % (30 * 75) == 0)
     cdio_debug ("reading %2.2d:%2.2d:%2.2d",
-	       msf->cdmsf_min0, msf->cdmsf_sec0, msf->cdmsf_frame0);
+		solaris_msf.cdmsf_min0, solaris_msf.cdmsf_sec0, 
+		solaris_msf.cdmsf_frame0);
   
   p_env->gen.ioctls_debugged++;
   
@@ -660,6 +661,10 @@ get_track_format_solaris(void *p_user_data, track_t i_track)
 {
   _img_private_t *p_env = p_user_data;
   
+  if ( !p_env ) return TRACK_FORMAT_ERROR;
+  if (!p_env->gen.init) init_solaris(p_env);
+  if (!p_env->gen.toc_init) read_toc_solaris (p_user_data) ;
+
   if ( (i_track > p_env->gen.i_tracks+p_env->gen.i_first_track) 
        || i_track < p_env->gen.i_first_track)
     return TRACK_FORMAT_ERROR;
@@ -694,6 +699,7 @@ _cdio_get_track_green(void *p_user_data, track_t i_track)
 {
   _img_private_t *p_env = p_user_data;
   
+  if ( !p_env ) return false;
   if (!p_env->gen.init) init_solaris(p_env);
   if (!p_env->gen.toc_init) read_toc_solaris (p_env) ;
 
@@ -736,9 +742,9 @@ _cdio_get_track_msf(void *p_user_data, track_t i_track, msf_t *msf)
     return false;
   } else {
     struct cdrom_tocentry *msf0 = &p_env->tocent[i_track-1];
-    msf->m = to_bcd8(msf0->cdte_addr.msf.minute);
-    msf->s = to_bcd8(msf0->cdte_addr.msf.second);
-    msf->f = to_bcd8(msf0->cdte_addr.msf.frame);
+    msf->m = cdio_to_bcd8(msf0->cdte_addr.msf.minute);
+    msf->s = cdio_to_bcd8(msf0->cdte_addr.msf.second);
+    msf->f = cdio_to_bcd8(msf0->cdte_addr.msf.frame);
     return true;
   }
 }
@@ -767,7 +773,7 @@ cdio_get_devices_solaris (void)
   char volpath[256];
   struct stat st;
   char **drives = NULL;
-  int num_files=0;
+  unsigned int i_files=0;
 #ifdef HAVE_GLOB_H
   unsigned int i;
   glob_t globbuf;
@@ -782,9 +788,9 @@ cdio_get_devices_solaris (void)
     if(S_ISDIR(st.st_mode)) {
       sprintf(volpath, "%s/s0", globbuf.gl_pathv[i]);
       if(stat(volpath, &st) == 0)
-        cdio_add_device_list(&drives, volpath, &num_files);
+        cdio_add_device_list(&drives, volpath, &i_files);
 	}else
-      cdio_add_device_list(&drives, globbuf.gl_pathv[i], &num_files);
+      cdio_add_device_list(&drives, globbuf.gl_pathv[i], &i_files);
   }
   globfree(&globbuf);
 #else
@@ -793,12 +799,12 @@ cdio_get_devices_solaris (void)
     if(S_ISDIR(st.st_mode)) {
       sprintf(volpath, "%s/s0", DEFAULT_CDIO_DEVICE);
       if(stat(volpath, &st) == 0)
-        cdio_add_device_list(&drives, volpath, &num_files);
+        cdio_add_device_list(&drives, volpath, &i_files);
     }else
-      cdio_add_device_list(&drives, DEFAULT_CDIO_DEVICE, &num_files);
+      cdio_add_device_list(&drives, DEFAULT_CDIO_DEVICE, &i_files);
   }
 #endif /*HAVE_GLOB_H*/
-  cdio_add_device_list(&drives, NULL, &num_files);
+  cdio_add_device_list(&drives, NULL, &i_files);
   return drives;
 #endif /*HAVE_SOLARIS_CDROM*/
 }
@@ -828,34 +834,35 @@ cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
   _img_private_t *_data;
   char *psz_source;
 
-  cdio_funcs _funcs = {
-    .eject_media        = eject_media_solaris,
-    .free               = cdio_generic_free,
-    .get_arg            = get_arg_solaris,
-    .get_cdtext         = get_cdtext_generic,
-    .get_default_device = cdio_get_default_device_solaris,
-    .get_devices        = cdio_get_devices_solaris,
-    .get_discmode       = get_discmode_solaris,
-    .get_drive_cap      = scsi_mmc_get_drive_cap_generic,
-    .get_first_track_num= get_first_track_num_generic,
-    .get_mcn            = scsi_mmc_get_mcn_generic,
-    .get_num_tracks     = get_num_tracks_generic,
-    .get_track_format   = get_track_format_solaris,
-    .get_track_green    = _cdio_get_track_green,
-    .get_track_lba      = NULL, /* This could be implemented if need be. */
-    .get_track_msf      = _cdio_get_track_msf,
-    .lseek              = cdio_generic_lseek,
-    .read               = cdio_generic_read,
-    .read_audio_sectors = _read_audio_sectors_solaris,
-    .read_mode1_sector  = _read_mode1_sector_solaris,
-    .read_mode1_sectors = _read_mode1_sectors_solaris,
-    .read_mode2_sector  = _read_mode2_sector_solaris,
-    .read_mode2_sectors = _read_mode2_sectors_solaris,
-    .read_toc           = read_toc_solaris,
-    .run_scsi_mmc_cmd   = run_scsi_cmd_solaris,
-    .stat_size          = _cdio_stat_size,
-    .set_arg            = _set_arg_solaris
-  };
+  cdio_funcs _funcs;
+
+  _funcs.eject_media        = eject_media_solaris;
+  _funcs.free               = cdio_generic_free;
+  _funcs.get_arg            = get_arg_solaris;
+  _funcs.get_cdtext         = get_cdtext_generic;
+  _funcs.get_default_device = cdio_get_default_device_solaris;
+  _funcs.get_devices        = cdio_get_devices_solaris;
+  _funcs.get_discmode       = get_discmode_solaris;
+  _funcs.get_drive_cap      = scsi_mmc_get_drive_cap_generic;
+  _funcs.get_first_track_num= get_first_track_num_generic;
+  _funcs.get_hwinfo         = NULL;
+  _funcs.get_mcn            = scsi_mmc_get_mcn_generic,
+  _funcs.get_num_tracks     = get_num_tracks_generic;
+  _funcs.get_track_format   = get_track_format_solaris;
+  _funcs.get_track_green    = _cdio_get_track_green;
+  _funcs.get_track_lba      = NULL; /* This could be implemented if need be. */
+  _funcs.get_track_msf      = _cdio_get_track_msf;
+  _funcs.lseek              = cdio_generic_lseek;
+  _funcs.read               = cdio_generic_read;
+  _funcs.read_audio_sectors = _read_audio_sectors_solaris;
+  _funcs.read_mode1_sector  = _read_mode1_sector_solaris;
+  _funcs.read_mode1_sectors = _read_mode1_sectors_solaris;
+  _funcs.read_mode2_sector  = _read_mode2_sector_solaris;
+  _funcs.read_mode2_sectors = _read_mode2_sectors_solaris;
+  _funcs.read_toc           = read_toc_solaris;
+  _funcs.run_scsi_mmc_cmd   = run_scsi_cmd_solaris;
+  _funcs.stat_size          = _cdio_stat_size;
+  _funcs.set_arg            = _set_arg_solaris;
 
   _data                 = _cdio_malloc (sizeof (_img_private_t));
 
@@ -877,7 +884,7 @@ cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
     else {
       /* The below would be okay if all device drivers worked this way. */
 #if 0
-      cdio_info ("source %s is a not a device", psz_orig_source);
+      cdio_info ("source %s is not a device", psz_orig_source);
 #endif
       return NULL;
     }
