@@ -1,5 +1,5 @@
 /*
-    $Id: bincue.c,v 1.41 2004/08/16 02:04:08 rocky Exp $
+    $Id: bincue.c,v 1.47 2004/11/07 21:13:10 rocky Exp $
 
     Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
@@ -26,7 +26,7 @@
    (*.cue).
 */
 
-static const char _rcsid[] = "$Id: bincue.c,v 1.41 2004/08/16 02:04:08 rocky Exp $";
+static const char _rcsid[] = "$Id: bincue.c,v 1.47 2004/11/07 21:13:10 rocky Exp $";
 
 #include "image.h"
 #include "cdio_assert.h"
@@ -35,6 +35,7 @@ static const char _rcsid[] = "$Id: bincue.c,v 1.41 2004/08/16 02:04:08 rocky Exp
 
 #include <cdio/logging.h>
 #include <cdio/util.h>
+#include <cdio/version.h>
 
 #ifdef HAVE_STDIO_H
 #include <stdio.h>
@@ -56,15 +57,7 @@ static const char _rcsid[] = "$Id: bincue.c,v 1.41 2004/08/16 02:04:08 rocky Exp
 #endif
 #include <ctype.h>
 
-/* FIXME: should put in a common definition somewhere. */
-#ifdef HAVE_MEMSET
-#define BZERO(ptr, size) memset(ptr, 0, size)
-#elif  HAVE_BZERO
-#define BZERO(ptr, size) bzero(ptr, size)
-#else 
-  Error -- you need either memset or bzero
-#endif
-
+#include "portable.h"
 /* reader */
 
 #define DEFAULT_CDIO_DEVICE "videocd.bin"
@@ -834,18 +827,18 @@ _read_audio_sectors_bincue (void *user_data, void *data, lsn_t lsn,
  */
 static int
 _read_mode1_sector_bincue (void *user_data, void *data, lsn_t lsn, 
-			 bool b_form2)
+			   bool b_form2)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = user_data;
   int ret;
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
   int blocksize = CDIO_CD_FRAMESIZE_RAW;
 
-  ret = cdio_stream_seek (env->gen.data_source, lsn * blocksize, SEEK_SET);
+  ret = cdio_stream_seek (p_env->gen.data_source, lsn * blocksize, SEEK_SET);
   if (ret!=0) return ret;
 
   /* FIXME: Not completely sure the below is correct. */
-  ret = cdio_stream_read (env->gen.data_source, buf, CDIO_CD_FRAMESIZE_RAW, 1);
+  ret = cdio_stream_read (p_env->gen.data_source, buf, CDIO_CD_FRAMESIZE_RAW, 1);
   if (ret==0) return ret;
 
   memcpy (data, buf + CDIO_CD_SYNC_SIZE + CDIO_CD_HEADER_SIZE, 
@@ -863,13 +856,13 @@ static int
 _read_mode1_sectors_bincue (void *user_data, void *data, lsn_t lsn, 
 			    bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = user_data;
   int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
   for (i = 0; i < nblocks; i++) {
-    if ( (retval = _read_mode1_sector_bincue (env, 
+    if ( (retval = _read_mode1_sector_bincue (p_env, 
 					    ((char *)data) + (blocksize * i),
 					    lsn + i, b_form2)) )
       return retval;
@@ -885,7 +878,7 @@ static int
 _read_mode2_sector_bincue (void *user_data, void *data, lsn_t lsn, 
 			 bool b_form2)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = user_data;
   int ret;
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
 
@@ -897,10 +890,10 @@ _read_mode2_sector_bincue (void *user_data, void *data, lsn_t lsn,
 
   int blocksize = CDIO_CD_FRAMESIZE_RAW;
 
-  ret = cdio_stream_seek (env->gen.data_source, lsn * blocksize, SEEK_SET);
+  ret = cdio_stream_seek (p_env->gen.data_source, lsn * blocksize, SEEK_SET);
   if (ret!=0) return ret;
 
-  ret = cdio_stream_read (env->gen.data_source, buf, CDIO_CD_FRAMESIZE_RAW, 1);
+  ret = cdio_stream_read (p_env->gen.data_source, buf, CDIO_CD_FRAMESIZE_RAW, 1);
   if (ret==0) return ret;
 
 
@@ -923,13 +916,13 @@ static int
 _read_mode2_sectors_bincue (void *user_data, void *data, lsn_t lsn, 
 			    bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = user_data;
   int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
   for (i = 0; i < nblocks; i++) {
-    if ( (retval = _read_mode2_sector_bincue (env, 
+    if ( (retval = _read_mode2_sector_bincue (p_env, 
 					    ((char *)data) + (blocksize * i),
 					    lsn + i, b_form2)) )
       return retval;
@@ -973,6 +966,16 @@ cdio_get_default_device_bincue(void)
   return drive;
 }
 
+static bool
+get_hwinfo_bincue ( const CdIo *p_cdio, /*out*/ cdio_hwinfo_t *hw_info)
+{
+  strcpy(hw_info->psz_vendor, "libcdio");
+  strcpy(hw_info->psz_model, "CDRWIN");
+  strcpy(hw_info->psz_revision, CDIO_VERSION);
+  return true;
+  
+}
+
 /*!
   Return the number of tracks in the current medium.
   CDIO_INVALID_TRACK is returned on error.
@@ -980,12 +983,12 @@ cdio_get_default_device_bincue(void)
 static track_format_t
 _get_track_format_bincue(void *user_data, track_t i_track) 
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = user_data;
   
-  if (i_track > env->gen.i_tracks || i_track == 0) 
+  if (i_track > p_env->gen.i_tracks || i_track == 0) 
     return TRACK_FORMAT_ERROR;
 
-  return env->tocent[i_track-env->gen.i_first_track].track_format;
+  return p_env->tocent[i_track-p_env->gen.i_first_track].track_format;
 }
 
 /*!
@@ -999,14 +1002,14 @@ _get_track_format_bincue(void *user_data, track_t i_track)
 static bool
 _get_track_green_bincue(void *user_data, track_t i_track) 
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = user_data;
   
-  if ( NULL == env || 
-       ( i_track < env->gen.i_first_track
-	 || i_track >= env->gen.i_tracks + env->gen.i_first_track ) )
+  if ( NULL == p_env || 
+       ( i_track < p_env->gen.i_first_track
+	 || i_track >= p_env->gen.i_tracks + p_env->gen.i_first_track ) )
     return false;
 
-  return env->tocent[i_track-env->gen.i_first_track].track_green;
+  return p_env->tocent[i_track-p_env->gen.i_first_track].track_green;
 }
 
 /*!  
@@ -1019,12 +1022,12 @@ _get_track_green_bincue(void *user_data, track_t i_track)
 static lba_t
 _get_lba_track_bincue(void *user_data, track_t i_track)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = user_data;
 
-  if (i_track == CDIO_CDROM_LEADOUT_TRACK) i_track = env->gen.i_tracks+1;
+  if (i_track == CDIO_CDROM_LEADOUT_TRACK) i_track = p_env->gen.i_tracks+1;
 
-  if (i_track <= env->gen.i_tracks + env->gen.i_first_track && i_track != 0) {
-    return env->tocent[i_track-env->gen.i_first_track].start_lba;
+  if (i_track <= p_env->gen.i_tracks + p_env->gen.i_first_track && i_track != 0) {
+    return p_env->tocent[i_track-p_env->gen.i_first_track].start_lba;
   } else 
     return CDIO_INVALID_LBA;
 }
@@ -1055,16 +1058,17 @@ cdio_is_cuefile(const char *psz_cue_name)
       if (parse_cuefile(NULL, psz_cue_name))
 	return psz_bin_name;
       else 
-	return NULL;
+	goto error;
     } 
     else if (psz_cue_name[i]=='C' && psz_cue_name[i+1]=='U' && psz_cue_name[i+2]=='E') {
       psz_bin_name[i++]='B'; psz_bin_name[i++]='I'; psz_bin_name[i++]='N';
       if (parse_cuefile(NULL, psz_cue_name))
 	return psz_bin_name;
       else 
-	return NULL;
+	goto error;
     }
   }
+ error:
   free(psz_bin_name);
   return NULL;
 }
@@ -1139,49 +1143,52 @@ cdio_open_cue (const char *psz_cue_name)
   CdIo *ret;
   _img_private_t *_data;
   char *psz_bin_name;
-
-  cdio_funcs _funcs = {
-    .eject_media        = _eject_media_image,
-    .free               = _free_image,
-    .get_arg            = _get_arg_image,
-    .get_cdtext         = get_cdtext_generic,
-    .get_devices        = cdio_get_devices_bincue,
-    .get_default_device = cdio_get_default_device_bincue,
-    .get_discmode       = _get_discmode_image,
-    .get_drive_cap      = _get_drive_cap_image,
-    .get_first_track_num= _get_first_track_num_image,
-    .get_mcn            = _get_mcn_image,
-    .get_num_tracks     = _get_num_tracks_image,
-    .get_track_format   = _get_track_format_bincue,
-    .get_track_green    = _get_track_green_bincue,
-    .get_track_lba      = _get_lba_track_bincue, 
-    .get_track_msf      = _get_track_msf_image,
-    .lseek              = _lseek_bincue,
-    .read               = _read_bincue,
-    .read_audio_sectors = _read_audio_sectors_bincue,
-    .read_mode1_sector  = _read_mode1_sector_bincue,
-    .read_mode1_sectors = _read_mode1_sectors_bincue,
-    .read_mode2_sector  = _read_mode2_sector_bincue,
-    .read_mode2_sectors = _read_mode2_sectors_bincue,
-    .set_arg            = _set_arg_image,
-    .stat_size          = _stat_size_bincue
-  };
-
+  
+  cdio_funcs _funcs;
+  
+  memset( &_funcs, 0, sizeof(_funcs) );
+  
+  _funcs.eject_media        = _eject_media_image;
+  _funcs.free               = _free_image;
+  _funcs.get_arg            = _get_arg_image;
+  _funcs.get_cdtext         = get_cdtext_generic;
+  _funcs.get_devices        = cdio_get_devices_bincue;
+  _funcs.get_default_device = cdio_get_default_device_bincue;
+  _funcs.get_discmode       = _get_discmode_image;
+  _funcs.get_drive_cap      = _get_drive_cap_image;
+  _funcs.get_first_track_num= _get_first_track_num_image;
+  _funcs.get_hwinfo         = get_hwinfo_bincue;
+  _funcs.get_mcn            = _get_mcn_image;
+  _funcs.get_num_tracks     = _get_num_tracks_image;
+  _funcs.get_track_format   = _get_track_format_bincue;
+  _funcs.get_track_green    = _get_track_green_bincue;
+  _funcs.get_track_lba      = _get_lba_track_bincue;
+  _funcs.get_track_msf      = _get_track_msf_image;
+  _funcs.lseek              = _lseek_bincue;
+  _funcs.read               = _read_bincue;
+  _funcs.read_audio_sectors = _read_audio_sectors_bincue;
+  _funcs.read_mode1_sector  = _read_mode1_sector_bincue;
+  _funcs.read_mode1_sectors = _read_mode1_sectors_bincue;
+  _funcs.read_mode2_sector  = _read_mode2_sector_bincue;
+  _funcs.read_mode2_sectors = _read_mode2_sectors_bincue;
+  _funcs.set_arg            = _set_arg_image;
+  _funcs.stat_size          = _stat_size_bincue;
+  
   if (NULL == psz_cue_name) return NULL;
-
+  
   _data                 = _cdio_malloc (sizeof (_img_private_t));
   _data->gen.init       = false;
   _data->psz_cue_name   = NULL;
-
+  
   ret = cdio_new ((void *)_data, &_funcs);
-
+  
   if (ret == NULL) {
     free(_data);
     return NULL;
   }
-
+  
   psz_bin_name = cdio_is_cuefile(psz_cue_name);
-
+  
   if (NULL == psz_bin_name) {
     cdio_error ("source name %s is not recognized as a CUE file", 
 		psz_cue_name);
@@ -1190,7 +1197,7 @@ cdio_open_cue (const char *psz_cue_name)
   _set_arg_image (_data, "cue", psz_cue_name);
   _set_arg_image (_data, "source", psz_bin_name);
   free(psz_bin_name);
-
+  
   if (_init_bincue(_data)) {
     return ret;
   } else {
