@@ -1,5 +1,5 @@
 /*
-    $Id: cd-info.c,v 1.89 2004/09/02 01:33:57 rocky Exp $
+    $Id: cd-info.c,v 1.100 2004/11/19 22:10:21 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 1996, 1997, 1998  Gerd Knorr <kraxel@bytesex.org>
@@ -26,6 +26,7 @@
 */
 
 #include "util.h"
+#include <stdarg.h>
 
 #ifdef HAVE_CDDB
 #include <cddb/cddb.h>
@@ -37,6 +38,8 @@
 #include <libvcd/info.h>
 #endif
 
+#include <cdio/bytesex.h>
+#include <cdio/ds.h>
 #include <cdio/util.h>
 #include <cdio/cd_types.h>
 #include <cdio/cdtext.h>
@@ -44,8 +47,6 @@
 #include <cdio/scsi_mmc.h>
 
 #include "cdio_assert.h"
-#include "bytesex.h"
-#include "ds.h"
 #include "iso9660_private.h"
 
 #include <fcntl.h>
@@ -59,13 +60,8 @@
  
 #include <errno.h>
 
-#if 0
-#define STRONG "\033[1m"
-#define NORMAL "\033[0m"
-#else
 #define STRONG "__________________________________\n"
 #define NORMAL ""
-#endif
 
 #if CDIO_IOCTL_FINISHED
 struct cdrom_multisession  ms;
@@ -98,6 +94,7 @@ struct arguments
   int            version_only;
   int            silent;
   int            no_header;
+  int            no_joliet;
   int            print_iso9660;
   int            list_drives;
   source_image_t source_image;
@@ -193,6 +190,12 @@ parse_options (int argc, const char *argv[])
     {"cue-file", 'c', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_CUE, "set \"cue\" CD-ROM disk image file as source", "FILE"},
     
+    {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
+     OP_SOURCE_NRG, "set Nero CD-ROM disk image file as source", "FILE"},
+    
+    {"toc-file", 't', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
+     OP_SOURCE_CDRDAO, "set cdrdao CD-ROM disk image file as source", "FILE"},
+    
     {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
      OP_SOURCE_AUTO,
      "set source and determine if \"bin\" image or device", "FILE"},
@@ -209,12 +212,11 @@ parse_options (int argc, const char *argv[])
     
     {"no-header", '\0', POPT_ARG_NONE, &opts.no_header, 
      0, "Don't display header and copyright (for regression testing)"},
-    
-    {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
-     OP_SOURCE_NRG, "set Nero CD-ROM disk image file as source", "FILE"},
-    
-    {"toc-file", 't', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &psz_my_source, 
-     OP_SOURCE_CDRDAO, "set cdrdao CD-ROM disk image file as source", "FILE"},
+
+#ifdef HAVE_JOLIET    
+    {"no-joliet", '\0', POPT_ARG_NONE, &opts.no_joliet, 
+     0, "Don't use Joliet extensions"},
+#endif /*HAVE_JOLIET*/
     
     {"quiet",       'q', POPT_ARG_NONE, &opts.silent, 0,
      "Don't produce warning output" },
@@ -238,11 +240,10 @@ parse_options (int argc, const char *argv[])
     case OP_SOURCE_NRG: 
     case OP_SOURCE_DEVICE: 
       if (opts.source_image != IMAGE_UNKNOWN) {
-	fprintf(stderr, 
-		"%s: another source type option given before.\n", 
-		program_name);
-	fprintf(stderr, "%s: give only one source type option.\n", 
-		program_name);
+	report(stderr, "%s: another source type option given before.\n", 
+		   program_name);
+	report(stderr, "%s: give only one source type option.\n", 
+		   program_name);
 	break;
       } 
 
@@ -285,10 +286,9 @@ parse_options (int argc, const char *argv[])
     const char *remaining_arg = poptGetArg(optCon);
     if ( remaining_arg != NULL) {
       if (opts.source_image != IMAGE_UNKNOWN) {
-	fprintf (stderr, 
-		 "%s: Source '%s' given as an argument of an option and as "
-		 "unnamed option '%s'\n", 
-		 program_name, psz_my_source, remaining_arg);
+	report(stderr, "%s: Source '%s' given as an argument of an option and as "
+		   "unnamed option '%s'\n", 
+	       program_name, psz_my_source, remaining_arg);
 	poptFreeContext(optCon);
 	free(program_name);
 	exit (EXIT_FAILURE);
@@ -300,9 +300,8 @@ parse_options (int argc, const char *argv[])
 	source_name = strdup(remaining_arg);
       
       if ( (poptGetArgs(optCon)) != NULL) {
-	fprintf (stderr, 
-		 "%s: Source specified in previously %s and %s\n", 
-		 program_name, psz_my_source, remaining_arg);
+	report(stderr, "%s: Source specified in previously %s and %s\n", 
+		   program_name, psz_my_source, remaining_arg);
 	poptFreeContext(optCon);
 	free(program_name);
 	exit (EXIT_FAILURE);
@@ -338,7 +337,7 @@ cddb_dec_digit_sum(int n)
 static inline unsigned int
 msf_seconds(msf_t *msf) 
 {
-  return from_bcd8(msf->m)*60 + from_bcd8(msf->s);
+  return cdio_from_bcd8(msf->m)*CDIO_CD_SECS_PER_MIN + cdio_from_bcd8(msf->s);
 }
 
 /* 
@@ -396,7 +395,7 @@ _log_handler (cdio_log_level_t level, const char message[])
 
 static void 
 print_cdtext_track_info(CdIo *p_cdio, track_t i_track, const char *message) {
-  const cdtext_t *cdtext = cdio_get_cdtext(p_cdio, 0);
+  const cdtext_t *cdtext = cdio_get_cdtext(p_cdio, i_track);
 
   if (NULL != cdtext) {
     cdtext_field_t i;
@@ -432,7 +431,7 @@ print_cddb_info(CdIo *p_cdio, track_t i_tracks, track_t i_first_track) {
   cddb_disc_t *disc = NULL;
 
   if (!conn) {
-    fprintf(stderr, "%s: unable to initialize libcddb\n", program_name);
+    report(stderr,  "%s: unable to initialize libcddb\n", program_name);
     goto cddb_destroy;
   }
 
@@ -464,7 +463,7 @@ print_cddb_info(CdIo *p_cdio, track_t i_tracks, track_t i_first_track) {
     
   disc = cddb_disc_new();
   if (!disc) {
-    fprintf(stderr, "%s: unable to create CDDB disc structure", program_name);
+    report(stderr, "%s: unable to create CDDB disc structure", program_name);
     goto cddb_destroy;
   }
   for(i = 0; i < i_tracks; i++) {
@@ -478,8 +477,8 @@ print_cddb_info(CdIo *p_cdio, track_t i_tracks, track_t i_first_track) {
     / CDIO_CD_FRAMES_PER_SEC;
 
   if (!cddb_disc_calc_discid(disc)) {
-    fprintf(stderr, "%s: libcddb calc discid failed.\n", 
-	    program_name);
+    report(stderr, "%s: libcddb calc discid failed.\n", 
+	       program_name);
     goto cddb_destroy;
   }
 
@@ -506,51 +505,55 @@ print_cddb_info(CdIo *p_cdio, track_t i_tracks, track_t i_first_track) {
 static void 
 print_vcd_info(driver_id_t driver) {
   vcdinfo_open_return_t open_rc;
-  vcdinfo_obj_t *obj;
-  open_rc = vcdinfo_open(&obj, &source_name, driver, NULL);
+  vcdinfo_obj_t *p_vcd = NULL;
+  open_rc = vcdinfo_open(&p_vcd, &source_name, driver, NULL);
   switch (open_rc) {
   case VCDINFO_OPEN_VCD: 
-    if (vcdinfo_get_format_version (obj) == VCD_TYPE_INVALID) {
-      fprintf(stderr, "VCD format detection failed");
-      vcdinfo_close(obj);
+    if (vcdinfo_get_format_version (p_vcd) == VCD_TYPE_INVALID) {
+      report(stderr, "VCD format detection failed");
+      vcdinfo_close(p_vcd);
       return;
     }
-    fprintf (stdout, "Format      : %s\n", 
-	     vcdinfo_get_format_version_str(obj));
-    fprintf (stdout, "Album       : `%.16s'\n",  vcdinfo_get_album_id(obj));
-    fprintf (stdout, "Volume count: %d\n",   vcdinfo_get_volume_count(obj));
-    fprintf (stdout, "volume number: %d\n",  vcdinfo_get_volume_num(obj));
+    report (stdout, "Format      : %s\n", 
+	    vcdinfo_get_format_version_str(p_vcd));
+    report (stdout, "Album       : `%.16s'\n",  vcdinfo_get_album_id(p_vcd));
+    report (stdout, "Volume count: %d\n",   vcdinfo_get_volume_count(p_vcd));
+    report (stdout, "volume number: %d\n",  vcdinfo_get_volume_num(p_vcd));
 
     break;
   case VCDINFO_OPEN_ERROR:
-    fprintf (stderr, "Error in Video CD opening of %s\n", source_name);
-    return;
+    report( stderr, "Error in Video CD opening of %s\n", source_name );
     break;
   case VCDINFO_OPEN_OTHER:
-    fprintf (stderr, "Even though we thought this was a Video CD, "
-	     " further inspection says it is not.\n");
+    report( stderr, "Even though we thought this was a Video CD, "
+	    " further inspection says it is not.\n" );
     break;
   }
-  vcdinfo_close(obj);
+  if (p_vcd) vcdinfo_close(p_vcd);
     
 }
 #endif 
 
 static void
-print_iso9660_recurse (const CdIo *p_cdio, const char pathname[], 
+print_iso9660_recurse (CdIo *p_cdio, const char pathname[], 
 		       cdio_fs_anal_t fs, 
 		       bool b_mode2)
 {
   CdioList *entlist;
   CdioList *dirlist =  _cdio_list_new ();
   CdioListNode *entnode;
+  uint8_t i_joliet_level;
+
+  i_joliet_level = (opts.no_joliet) 
+    ? 0
+    : cdio_get_joliet_level(p_cdio);
 
   entlist = iso9660_fs_readdir (p_cdio, pathname, b_mode2);
     
   printf ("%s:\n", pathname);
 
   if (NULL == entlist) {
-    fprintf (stderr, "Error getting above directory information\n");
+    report( stderr, "Error getting above directory information\n" );
     return;
   }
 
@@ -565,7 +568,7 @@ print_iso9660_recurse (const CdIo *p_cdio, const char pathname[],
 #define DATESTR_SIZE 30
       char date_str[DATESTR_SIZE];
 
-      iso9660_name_translate(iso_name, translated_name);
+      iso9660_name_translate_ext(iso_name, translated_name, i_joliet_level);
       
       snprintf (_fullname, sizeof (_fullname), "%s%s", pathname, 
 		iso_name);
@@ -614,55 +617,39 @@ print_iso9660_recurse (const CdIo *p_cdio, const char pathname[],
   _cdio_list_free (dirlist, true);
 }
 
-static bool
-read_iso9660_pvd(const CdIo *p_cdio, track_format_t track_format, /*out*/
-		 iso9660_pvd_t *p_pvd) 		 {
-  
-  switch (track_format) {
-  case TRACK_FORMAT_CDI:
-  case TRACK_FORMAT_XA:
-    if (0 != cdio_read_mode2_sector (p_cdio, p_pvd, ISO_PVD_SECTOR, false))
-      return false;
-    break;
-  case TRACK_FORMAT_DATA:
-    if (0 != cdio_read_mode1_sector (p_cdio, p_pvd, ISO_PVD_SECTOR, false))
-      return false;
-    break;
-  case TRACK_FORMAT_AUDIO: 
-  case TRACK_FORMAT_PSX: 
-  case TRACK_FORMAT_ERROR: 
-  default:
-    return false;
-  }
-  return true;
-}
-		 
-
 static void
-print_iso9660_fs (const CdIo *p_cdio, cdio_fs_anal_t fs, 
+print_iso9660_fs (CdIo *p_cdio, cdio_fs_anal_t fs, 
 		  track_format_t track_format)
 {
-  iso9660_pvd_t pvd;
   bool b_mode2 = false;
+  iso_extension_mask_t iso_extension_mask = ISO_EXTENSION_ALL;
 
   if (fs & CDIO_FS_ANAL_XA) track_format = TRACK_FORMAT_XA;
 
-  if ( !read_iso9660_pvd(p_cdio, track_format, &pvd) ) 
+  if (opts.no_joliet) {
+    iso_extension_mask &= ~ISO_EXTENSION_JOLIET;
+  }
+
+  if ( !iso9660_fs_read_superblock(p_cdio, iso_extension_mask) ) 
     return;
   
   b_mode2 = ( TRACK_FORMAT_CDI == track_format 
 	       || TRACK_FORMAT_XA == track_format );
   
   {
-    const lsn_t extent = iso9660_get_root_lsn(&pvd);
-    
     printf ("ISO9660 filesystem\n");
-    printf (" root dir in PVD set to lsn %lu\n\n", (long unsigned) extent);
-    
     print_iso9660_recurse (p_cdio, "/", fs, b_mode2);
   }
 }
 
+#define print_vd_info(title, fn)		\
+  psz_str = fn(&pvd);				\
+  if (psz_str) {				\
+    report(stdout, title ": %s\n", psz_str);	\
+    free(psz_str);				\
+    psz_str = NULL;				\
+  }
+
 static void
 print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis, 
 	       cdio_fs_anal_t fs, int first_data, unsigned int num_audio, 
@@ -736,16 +723,15 @@ print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis,
     {
       iso9660_pvd_t pvd;
 
-      if ( read_iso9660_pvd(p_cdio, track_format, &pvd) ) {
-	fprintf(stdout, "Application: %s\n", 
-		iso9660_get_application_id(&pvd));
-	fprintf(stdout, "Preparer   : %s\n", iso9660_get_preparer_id(&pvd));
-	fprintf(stdout, "Publisher  : %s\n", iso9660_get_publisher_id(&pvd));
-	fprintf(stdout, "System     : %s\n", iso9660_get_system_id(&pvd));
-	fprintf(stdout, "Volume     : %s\n", iso9660_get_volume_id(&pvd));
-	fprintf(stdout, "Volume Set : %s\n", iso9660_get_volumeset_id(&pvd));
+      if ( iso9660_fs_read_pvd(p_cdio, &pvd) ) {
+	char *psz_str;
+	print_vd_info("Application", iso9660_get_application_id);
+	print_vd_info("Preparer   ", iso9660_get_preparer_id);
+	print_vd_info("Publisher  ", iso9660_get_publisher_id);
+	print_vd_info("System     ", iso9660_get_system_id);
+	print_vd_info("Volume     ", iso9660_get_volume_id);
+	print_vd_info("Volume Set ", iso9660_get_volumeset_id);
       }
-      
     }
     
     if (opts.print_iso9660) 
@@ -757,8 +743,8 @@ print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis,
   switch(CDIO_FSTYPE(fs)) {
   case CDIO_FS_UDF:
   case CDIO_FS_ISO_UDF:
-    fprintf(stdout, "UDF: version %x.%2.2x\n",
-	    cdio_iso_analysis.UDFVerMajor, cdio_iso_analysis.UDFVerMinor);
+    report(stdout, "UDF: version %x.%2.2x\n",
+	   cdio_iso_analysis.UDFVerMajor, cdio_iso_analysis.UDFVerMinor);
     break;
   default: ;
   }
@@ -816,6 +802,7 @@ init(void)
   opts.silent        = false;
   opts.list_drives   = false;
   opts.no_header     = false;
+  opts.no_joliet     = false;
   opts.no_device     = 0;
   opts.no_disc_mode  = 0;
   opts.debug_level   = 0;
@@ -847,21 +834,21 @@ int
 main(int argc, const char *argv[])
 {
 
-  CdIo               *p_cdio=NULL;
-  cdio_fs_anal_t      fs=CDIO_FS_AUDIO;
+  CdIo                *p_cdio=NULL;
+  cdio_fs_anal_t       fs=CDIO_FS_AUDIO;
   int i;
-  lsn_t               start_track_lsn;      /* lsn of first track */
-  lsn_t               data_start     =  0;  /* start of data area */
-  int                 ms_offset      =  0;
-  track_t             i_tracks       =  0;
-  track_t             i_first_track  =  0;
-  unsigned int        num_audio      =  0;  /* # of audio tracks */
-  unsigned int        num_data       =  0;  /* # of data tracks */
-  int                 first_data     = -1;  /* # of first data track */
-  int                 first_audio    = -1;  /* # of first audio track */
-  cdio_iso_analysis_t cdio_iso_analysis; 
+  lsn_t                start_track_lsn;      /* lsn of first track */
+  lsn_t                data_start     =  0;  /* start of data area */
+  int                  ms_offset      =  0;
+  track_t              i_tracks       =  0;
+  track_t              i_first_track  =  0;
+  unsigned int         num_audio      =  0;  /* # of audio tracks */
+  unsigned int         num_data       =  0;  /* # of data tracks */
+  int                  first_data     = -1;  /* # of first data track */
+  int                  first_audio    = -1;  /* # of first audio track */
+  cdio_iso_analysis_t  cdio_iso_analysis; 
   char                *media_catalog_number;  
-  discmode_t          discmode = CDIO_DISC_MODE_NO_INFO;
+  discmode_t           discmode = CDIO_DISC_MODE_NO_INFO;
       
   memset(&cdio_iso_analysis, 0, sizeof(cdio_iso_analysis));
   init();
@@ -1024,7 +1011,7 @@ main(int argc, const char *argv[])
     if (device_list) free(device_list);
   }
 
-  printf(STRONG "\n");
+  report(stdout, STRONG "\n");
   
 
   discmode = cdio_get_discmode(p_cdio);
@@ -1072,6 +1059,7 @@ main(int argc, const char *argv[])
       if (!opts.no_tracks)
 	printf("%3d: %8s  %06lu  leadout\n", (int) i, psz_msf,
 	       (long unsigned int) cdio_msf_to_lsn(&msf));
+      free(psz_msf);
       break;
     } else if (!opts.no_tracks) {
       printf("%3d: %8s  %06lu  %-5s %s\n", (int) i, psz_msf,
@@ -1079,6 +1067,7 @@ main(int argc, const char *argv[])
 	     track_format2str[cdio_get_track_format(p_cdio, i)],
 	     cdio_get_track_green(p_cdio, i)? "true" : "false");
     }
+    free(psz_msf);
     
     if (TRACK_FORMAT_AUDIO == cdio_get_track_format(p_cdio, i)) {
       num_audio++;
@@ -1108,7 +1097,7 @@ main(int argc, const char *argv[])
     
 #if CDIO_IOCTL_FINISHED
   if (!opts.no_ioctl) {
-    printf(STRONG "What ioctl's report...\n" NORMAL);
+    report(stdout, "What ioctl's report...\n");
   
 #ifdef CDROMMULTISESSION
     /* get multisession */
@@ -1151,7 +1140,7 @@ main(int argc, const char *argv[])
 #endif /*CDIO_IOCTL_FINISHED*/
   
   if (!opts.no_analysis) {
-    printf(STRONG "CD Analysis Report\n" NORMAL);
+    report(stdout, STRONG "CD Analysis Report\n" NORMAL);
     
     /* try to find out what sort of CD we have */
     if (0 == num_data) {
