@@ -1,5 +1,5 @@
 /*
-    $Id: iso-info.c,v 1.9 2004/06/19 19:15:15 rocky Exp $
+    $Id: iso-info.c,v 1.17 2004/11/04 10:08:23 rocky Exp $
 
     Copyright (C) 2004 Rocky Bernstein <rocky@panix.com>
 
@@ -23,8 +23,8 @@
 #include "util.h"
 #undef err_exit
 
-#define err_exit(fmt, args...) \
-  fprintf(stderr, "%s: "fmt, program_name, ##args); \
+#define err_exit(fmt, args...)			    \
+  report (stderr, "%s: "fmt, program_name, ##args); \
   iso9660_close(p_iso);				    \
   return(EXIT_FAILURE);
 
@@ -32,7 +32,9 @@
 # include "config.h"
 #endif
 #include <sys/types.h>
+#include <cdio/bytesex.h>
 #include <cdio/cdio.h>
+#include <cdio/ds.h>
 #include <cdio/iso9660.h>
 
 #include <stdio.h>
@@ -45,7 +47,6 @@
 
 #include <errno.h>
 
-#include "ds.h"
 
 #if 0
 #define STRONG "\033[1m"
@@ -64,6 +65,7 @@ struct arguments
   int            version_only;
   int            silent;
   int            no_header;
+  int            no_joliet;
   int            print_iso9660;
 } opts;
      
@@ -93,6 +95,11 @@ parse_options (int argc, const char *argv[])
     
     {"no-header", '\0', POPT_ARG_NONE, &opts.no_header, 
      0, "Don't display header and copyright (for regression testing)"},
+
+#ifdef HAVE_JOLIET    
+    {"no-joliet", '\0', POPT_ARG_NONE, &opts.no_joliet, 
+     0, "Don't use Joliet extensions"},
+#endif /*HAVE_JOLIET*/
     
     {"quiet",       'q', POPT_ARG_NONE, &opts.silent, 0,
      "Don't produce warning output" },
@@ -117,9 +124,8 @@ parse_options (int argc, const char *argv[])
     const char *remaining_arg = poptGetArg(optCon);
     if ( remaining_arg != NULL) {
       if ( (poptGetArgs(optCon)) != NULL) {
-	fprintf (stderr, 
-		 "%s: Source specified in previously %s and %s\n", 
-		 program_name, source_name, remaining_arg);
+	report( stderr, "%s: Source specified in previously %s and %s\n", 
+		    program_name, source_name, remaining_arg );
 	poptFreeContext(optCon);
 	exit (EXIT_FAILURE);
       }
@@ -151,18 +157,19 @@ _log_handler (cdio_log_level_t level, const char message[])
 }
 
 static void
-print_iso9660_recurse (iso9660_t *iso, const char pathname[])
+print_iso9660_recurse (iso9660_t *p_iso, const char pathname[])
 {
   CdioList *entlist;
   CdioList *dirlist =  _cdio_list_new ();
   CdioListNode *entnode;
+  uint8_t i_joliet_level = iso9660_ifs_get_joliet_level(p_iso);
 
-  entlist = iso9660_ifs_readdir (iso, pathname);
+  entlist = iso9660_ifs_readdir (p_iso, pathname);
     
   printf ("%s:\n", pathname);
 
   if (NULL == entlist) {
-    fprintf (stderr, "Error getting above directory information\n");
+    report( stderr, "Error getting above directory information\n" );
     return;
   }
 
@@ -177,7 +184,7 @@ print_iso9660_recurse (iso9660_t *iso, const char pathname[])
 #define DATESTR_SIZE 30
       char date_str[DATESTR_SIZE];
 
-      iso9660_name_translate(iso_name, translated_name);
+      iso9660_name_translate_ext(iso_name, translated_name, i_joliet_level);
       
       snprintf (_fullname, sizeof (_fullname), "%s%s", pathname, 
 		iso_name);
@@ -189,8 +196,8 @@ print_iso9660_recurse (iso9660_t *iso, const char pathname[])
           && strcmp (iso_name, ".."))
         _cdio_list_append (dirlist, strdup (_fullname));
 
-#if FINISHED
-      if (fs & CDIO_FS_ANAL_XA) {
+#if 1
+      if (iso9660_ifs_is_xa(p_iso)) {
 	printf ( "  %c %s %d %d [fn %.2d] [LSN %6lu] ",
 		 (statbuf->type == _STAT_DIR) ? 'd' : '-',
 		 iso9660_get_xa_attr_str (statbuf->xa.attributes),
@@ -222,7 +229,7 @@ print_iso9660_recurse (iso9660_t *iso, const char pathname[])
     {
       char *_fullname = _cdio_list_node_data (entnode);
 
-      print_iso9660_recurse (iso, _fullname);
+      print_iso9660_recurse (p_iso, _fullname);
     }
 
   _cdio_list_free (dirlist, true);
@@ -244,10 +251,18 @@ init(void)
   /* Default option values. */
   opts.silent        = false;
   opts.no_header     = false;
+  opts.no_joliet     = false;
   opts.debug_level   = 0;
   opts.print_iso9660 = 0;
 }
 
+#define print_vd_info(title, fn)	  \
+  if (fn(p_iso, &psz_str)) {		  \
+    printf(title ": %s\n", psz_str);	  \
+    free(psz_str);			  \
+    psz_str = NULL;			  \
+  }
+
 /* ------------------------------------------------------------------------ */
 
 int
@@ -255,6 +270,7 @@ main(int argc, const char *argv[])
 {
 
   iso9660_t           *p_iso=NULL;
+  iso_extension_mask_t iso_extension_mask = ISO_EXTENSION_ALL;
       
   init();
 
@@ -271,29 +287,30 @@ main(int argc, const char *argv[])
   }
 
   if (source_name==NULL) {
-    err_exit("%s: No input device given/found\n", program_name);
+    err_exit("No input device given/found%s\n", "");
   } 
 
-  p_iso = iso9660_open (source_name);
+  if (opts.no_joliet) {
+    iso_extension_mask &= ~ISO_EXTENSION_JOLIET;
+  }
+  
+  p_iso = iso9660_open_ext (source_name, iso_extension_mask);
 
   if (p_iso==NULL) {
     free(source_name);
-    err_exit("%s: Error in opening device driver\n", program_name);
+    err_exit("Error in opening ISO-9660 image%s\n", "");
   } 
 
   if (opts.silent == 0) {
-    iso9660_pvd_t pvd;
-
+    char *psz_str = NULL;
+    
     printf(STRONG "ISO 9660 image: %s\n", source_name);
-
-    if (iso9660_ifs_read_pvd(p_iso, &pvd)) {
-      printf("Application: %s\n", iso9660_get_application_id(&pvd));
-      printf("Preparer   : %s\n", iso9660_get_preparer_id(&pvd));
-      printf("Publisher  : %s\n", iso9660_get_publisher_id(&pvd));
-      printf("System     : %s\n", iso9660_get_system_id(&pvd));
-      printf("Volume     : %s\n", iso9660_get_volume_id(&pvd));
-      printf("Volume Set : %s\n", iso9660_get_volumeset_id(&pvd));
-    }
+    print_vd_info("Application", iso9660_ifs_get_application_id);
+    print_vd_info("Preparer   ", iso9660_ifs_get_preparer_id);
+    print_vd_info("Publisher  ", iso9660_ifs_get_publisher_id);
+    print_vd_info("System     ", iso9660_ifs_get_system_id);
+    print_vd_info("Volume     ", iso9660_ifs_get_volume_id);
+    print_vd_info("Volume Set ", iso9660_ifs_get_volumeset_id);
   }
   
   if (!opts.no_analysis) {
