@@ -1,5 +1,5 @@
 /*
-  $Id: util.h,v 1.7 2004/08/07 01:48:36 rocky Exp $
+  $Id: util.h,v 1.9 2004/11/06 09:16:04 rocky Exp $
 
   Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
   
@@ -38,6 +38,11 @@
 #endif
 #include <ctype.h>
 
+#ifdef HAVE_STDARG_H
+/* Get a definition for va_list.  */
+#include <stdarg.h>
+#endif
+
 #include <popt.h>
 /* Accomodate to older popt that doesn't support the "optional" flag */
 #ifndef POPT_ARGFLAG_OPTIONAL
@@ -69,13 +74,13 @@
 #if DEBUG
 #define dbg_print(level, s, args...) \
    if (opts.debug_level >= level) \
-     fprintf(stderr, "%s: "s, __func__ , ##args)
+     report(stderr, "%s: "s, __func__ , ##args)
 #else
 #define dbg_print(level, s, args...) 
 #endif
 
 #define err_exit(fmt, args...) \
-  fprintf(stderr, "%s: "fmt, program_name, ##args); \
+  report(stderr, "%s: "fmt, program_name, ##args); \
   myexit(p_cdio, EXIT_FAILURE)		     
   
 typedef enum
@@ -108,5 +113,8 @@ void print_drive_capabilities(cdio_drive_read_cap_t  p_read_cap,
 			      cdio_drive_write_cap_t p_write_cap,
 			      cdio_drive_misc_cap_t  p_misc_cap);
 
+/*! Common place for output routine. In some environments, like XBOX,
+  it may not be desireable to send output to stdout and stderr. */
+void report (FILE *stream, const char *psz_format, ...);
 
 #endif /* UTIL_H */
